%**********
% color_to_bw.m
%
% Routine to convert a color image into a black and
% white one based on a color scale included in the
% image and located in the rectangle with upper left
% corner x1,y1 and lower right corner x2,y2. x is the
% number of the row and y is the number of the column.
% If x1, y1, x2 and y2 are not provided, the function
% will ask the user to click on the corners of the scale.
% Provided are the three splitted channels on 8 bits:
% red (matr), green (matg), blue (matb). Returned
% matrix mat is the black and white image coded on
% unsigned char (0 to 255).
%
% Example of use:
%  mat=color_to_bw(matr, matg, matb, x1, y1, x2, y2);
%
% Adjustable parameter Number is used to decide the number of
% elements taken on each side to average the scale. If
% you want the scale to be kept unchanged, set Number to 0.
%
% This routine is useful to convert color scanned
% printed thermograms whose black and white version is
% not available. Splitted channel images are easily
% obtained from the original image in software such as
% Paint Shop Pro among others.
%
% Date : 08.06.2000
%
% Last modification: 18.07.2000
%
%**********

function mat=color_to_bw(matr, matg, matb, varargin);


Number=2;

% ***** Get the coordinates of the scale
if isempty(varargin)
	colormap(gray);
	imagesc(matr);	% display the image

	disp('Click on the upper left corner of the scale');
	[y1,x1]=ginput(1);	% upper left corner

	disp('Click on the lower right corner of the scale');
	[y2,x2]=ginput(1);	% lower right corner

	x1=round(x1);	% round towards nearest integer
	x2=round(x2);
	y1=round(y1);
	y2=round(y2);

else
	x1=varargin{1};
	y1=varargin{2};
	x2=varargin{3};
	y2=varargin{4};
end


% ***** Get the color scale
if y2-y1 > x2-x1	% horizontal scale

scale(1,:)=sum(matr(x1:x2,y1:y2))./(x2-x1+1);	% get the red color scale
scale(2,:)=sum(matg(x1:x2,y1:y2))./(x2-x1+1);	% get the green color scale
scale(3,:)=sum(matb(x1:x2,y1:y2))./(x2-x1+1);	% get the blue color scale

else	% vertical scale

scale(1,:)=sum(matr(x1:x2,y1:y2)')./(y2-y1+1);	% get the red color scale
scale(2,:)=sum(matg(x1:x2,y1:y2)')./(y2-y1+1);	% get the green color scale
scale(3,:)=sum(matb(x1:x2,y1:y2)')./(y2-y1+1);	% get the blue color scale

end

[srow scol]=size(scale);
scale(4,:)=mat_fl_to_uc(1:scol);	% vector 0-255, same length as scale

scale=scale';	% transpose to use the function sum


% ***** Improve the color scale
if Number > 0
	for col=1:scol

	first=col-Number;
	last=col+Number;

	if first < 1
 	   	first=1;
   	elseif last > scol
	   	last=scol;
   	end

   	total=last-first+1;
   	newscale(col,1:3)=sum(scale(first:last,1:3))./(total);
	% sum from first to last and divide by total to find average

	end
	scale(:,1:3)=newscale;	% save the result in scale
end


% ***** Convert the image
[mrow mcol]=size(matr);
mat=zeros(mrow,mcol);	% initialization

for i=1:mrow	% for all pixels in the image
for j=1:mcol

% find the position of the pixel in the scale (nearest absolute value)
vecred=abs(scale(:,1)-matr(i,j));
vecgreen=abs(scale(:,2)-matg(i,j));
vecblue=abs(scale(:,3)-matb(i,j));

vector=vecred+vecgreen+vecblue;
minvalue=min(vector);
f=find(vector==minvalue);

mat(i,j)=scale(f(1),4);	% store the corresponding black and white value in mat

end
end

clear matr;
clear matg;
clear matb;


% ***** Display the image
colormap(gray);
imagesc(mat);
colorbar;

