%**********
% defect.m
%
% Implementation of the automatic defect detection
% procedure on unsigned character image mat with
% parameter MND. Max_def is the maximum number of
% defects that can be found in an image. If parameter
% cross is set to 1, returned image mat1 is equal to
% mat with only crosses indicated at defect center.
% Any other value of cross returns ma1 as a map of
% defect(s) detected in mat (0 value at background
% and 1 value at defect location). mat_out is the
% matrix containing:
%	first column:	row of defect center
%	second column:	column of defect center
%	third column:	value of defect center
%
% Example of use:
%  [mat1, mat_out]=defect(mat,50,10,1);
%
% Adjustable parameters are:
% Border_width:	width in pixels of the border
%  around the image that is not searched for defects
% Cross_size:	width on each side of the cross in pixels
% Alpha:	used to find the minimum value of a defect
%
% Note: if you have the Image Processing Toolbox,
%  you may add a parameter (any number will do) after
%  cross to enable the use of functions from this
%  toolbox. You may obtain better results with these
%  functions.
% Example of use with the Image Processing Toolbox:
%  [mat1, mat_out]=defect(mat,50,10,1,1);
%	
%
% Date : 30.05.2000
%
% Last modification: 19.06.2000
%
%**********

function [mat1, varargout] = defect(mat,MND,Max_def,cross,varargin)


Border_width=4;	% minimum value is 1 pixel to have enough space for crosses
Cross_size=2;	% must be smaller or equal to Border_witdh, if =0 only a point
Alpha=1.0;	% parameter for the minimum value of a defect
% the equation is: min= Alpha*(mean_value+Alpha*std_dev) where mean_value
% is the mean value of mat and std_dev is the standard deviation of mat
% if the value of Alpha is too high, no defect will be found

% ***** Size of the matrix
[mrow mcol]=size(mat);
total=mrow*mcol;


% ***** Sort pixels of matrix mat
[mat1d_ord,ind1d]=sort(mat(:));

mat1d_ord=flipud(mat1d_ord);	% reverse order from largest to smallest
ind1d=flipud(ind1d);	% reverse order from largest to smallest

[x y]= ind2sub([mrow mcol],ind1d);	% get x,y coodinates from ind1d


% ***** Minimum value of a defect
mean_value=mean(mat(:));	% mean value of mat
std_dev=std(mat(:));	% standard deviation of mat
min_value=Alpha*(mean_value + Alpha*std_dev);	% minimum value of a defect
last_higher=max(find(mat1d_ord>min_value));


if isempty(last_higher)	% no defect in the image
   display('There is no defect');
   mat1=mat;
	varargout={[]};
else


% ***** Find defect centers
n_pix_ini(1)=1;	% initialization of first defect pixel
i=0;
j=1:total;
pixel=0;
total_limit=ones(total,1);

while (pixel<=last_higher) & (i<Max_def)
	i=i+1;
	xi(i)=x(n_pix_ini(i));
	yi(i)=y(n_pix_ini(i));
	distance=sqrt((x-xi(i)).^2+(y-yi(i)).^2);
	limit_dist=(distance > MND); 
% limit_dist = 0 if distance<d and 1 if distance>MND
	total_limit=total_limit & limit_dist;
% total_limit = 0 if limit_dist = 0 or distance<d for defect found before
   
	indice=(j > n_pix_ini(i))';
% indice=0 for pixels higher than n_pix_ini and 1 for pixels lower

	pixel=min(find(total_limit & indice));

	if isempty(pixel)
		break;
	elseif (pixel<=last_higher) & (i<Max_def)
		n_pix_ini(i+1)=pixel;
	end

end


% eliminate border around the image
up=find(xi<=Border_width);
down=find(mrow-xi<=Border_width);
left=find(yi<=Border_width);
right=find(mcol-yi<=Border_width);

n_pix_ini(up)=0;
n_pix_ini(down)=0;
n_pix_ini(left)=0;
n_pix_ini(right)=0;

f=find(n_pix_ini > 0);
n_pix_ini=n_pix_ini(f);
xi=xi(f);
yi=yi(f);

[r nbr]=size(f);

mat_out=[xi' yi' mat1d_ord(n_pix_ini)];
varargout={mat_out};


% ***** Parameter cross

if cross==1
% ***** Add crosses to mat
mat1=mat;
s=Cross_size;	% size of the cross

% ***** Draw the crosses on each defect center
for i=1:nbr

mat1(xi(i)-s:xi(i)+s,yi(i))=0;
mat1(xi(i),yi(i)-s:yi(i)+s)=0;

end


else
% ***** Threshold value of each defect

[val_histo,histo]=hist(mat(:),256);
for i=1:nbr

	% variation of the number of aglomerated pixels function
	% to the value of the thresholding

	agl_nbr=cumsum(fliplr(val_histo(histo < mat1d_ord(n_pix_ini(i))))); 	

	% computation of the angle formed by 3 points of the curb
	% distant of 20 pixels each

	ind=21:(size(agl_nbr,2)-20);	

	agl_nbr_n=256*(agl_nbr/max(agl_nbr));
	y1=abs(agl_nbr_n(ind+20)-agl_nbr_n(ind));
	y2=abs(agl_nbr_n(ind-20)-agl_nbr_n(ind));
	teta=pi+atan(y2/20)-atan(y1/20);

	pt_inflexion=max(find(teta == min(teta))+20);
	thres(i)=histo(pt_inflexion);

end

% ***** Extract image with defects
if isempty(varargin)
value=min(thres);
mat1=(mat>value);
mat1=median_filter(mat1,3);

else
for i=1:nbr
	im_thres=(mat > thres(i));

	% Use of the Image Processing Toolbox to clear up images
	label=bwlabel(im_thres,4);
	label2=(label == label(int16(xi(i)),int16(yi(i))));  
	im(:,:,i)=bwmorph(label2,'close');
end

mat1=(sum(im,3) ~= 0);

end	% if Toolbox

end	% if cross

end	% if no defect


% ***** Display the image
colormap(gray);
imagesc(mat1);
colorbar;
